/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    fcf_ent_load_alerted_entities.sas
| PRODUCT: SAS Compliance Solutions
| SYSTEM:  Windows, Unix
| DESC:    Creates the table fsk_alerted_entities in Knowledge Center.  This
|          table contains summary information for all entities with active 
|          alerts in seg_kc.fsk_alert.  The transactions count and aggregate
|          currency amount is intitialized to zero.  These values are loaded
|          after the fsk_entity_transaction table is created.
+------------------------------------------------------------------------------+
| USAGE: %fcf_ent_load_alerted_entity
+------------------------------------------------------------------------------+
| COMMENTS: INPUTS
|             None
|
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2016 - 2017 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/
%macro fcf_ent_load_alerted_entities;

/*------------------------------------------
 * Set error code to 0 and message to blank
 *------------------------------------------*/
 %let trans_rc = 0;
 %let _fcfmsg_=;

/*-------------------------------------------------
 * Save current routing information before
 * deleting and regenerating fsk_alerted_entities
 *------------------------------------------------*/
 %put NOTE: FCF: Saving current routing information prior to rebuilding the alerted_entity table;
  proc sql ;
     create table current_routing as
     select alerted_entity_level_code, alerted_entity_number, owner_userid
     from &seg_kc_libname..fsk_alerted_entity
     where alerted_entity_key ne -1
     order by alerted_entity_level_code, alerted_entity_number;
  quit;
  
  proc sql;
     delete from seg_kc.fsk_alerted_entity;
  quit;

/*---------------------------------------------------------------------------------------
 * Create a temporary table for entities with alerts and loading fsk_alerted_entity table
 *--------------------------------------------------------------------------------------*/
 %put NOTE: FCF: Creating a temporary table for entity with alerts and loading fsk_alerted_entity table;

 /*---------------------------------------------------------
  * Add a seed record in the fsk_alerted_entity table 
  *-------------------------------------------------------*/
 proc sql noprint;
    %put NOTE: FCF: Adding a seed record in the fsk_alerted_entity table;
    INSERT INTO &seg_kc_libname..fsk_alerted_entity(alerted_entity_key,alerted_entity_level_code,
                                          alerted_entity_number,alerted_entity_name,
                                          transactions_cnt,alerts_cnt,
                                          aggregate_amt,age_oldest_alert,
                                          risk_score_code,money_laundering_score,
                                          employee_ind,politically_exposed_person_ind, owner_userid,
                                          created_timestamp)
                                       VALUES(-1,'na','na','na',0,0,0,0,'0',0,NULL,NULL,'na',&fcf_start_datetime.);
 quit;

 %fcf_rcset(&sqlrc);
 %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
     %let _fcfmsg_=ERROR: FCF: Failed to add seed record to seg_kc.fsk_alerted_entity_table;
     %fcf_put_error;
     %goto MACRO_END;
 %end;

 /*------------------------------------------------------------------------------------
  * Find alerted_entity_number and alerted_entity_name for each alert_id in fsk_alert 
  * where alerted_entity_level_code is PTY
  * Note:  Since there is an employee_ind value for accounts, parties, and households
  *        that could be different for the same alerted entity, always use the value
  *        from the party_dim in the triage table.
  *----------------------------------------------------------------------------------*/

  /* Need to add the risk_classification value for parties            */
 /* Check to see if this value should come from RCP or CDD           */
 /* For RCP, use the risk_classification from fsc_party_dim          */
 /* For CDD, use the final_disposition_code from fsk_cdd_entity_fact */

 %fcf_get_property_value(property=fcf.riskScore.source,
                         output_property_value=RiskProcess,
                         output_status=property_status);

 %if &property_status ne 0 %then %do;
     %let _fcfmsg_=Warning: FCF: Missing property value to determine source system of risk classification. Set default of RCP;
     %fcf_put_error;
 %end;
 %let RiskProcess=&RiskProcess;

 %put NOTE:  Set Risk Classification value based on &RiskProcess;
 proc sql;

    connect to &dbflavor as &dbflavor (authdomain="&segkcAuthDomain." &segKCDBConnOpts);

    create table fsk_alert as
    select *
    from connection to &dbflavor

    (select alt.alert_id,
            alt.alerted_entity_level_code,
            alt.alerted_entity_key,
            alerted_entity_number, 
            pty.party_name as alerted_entity_name,
            %if &RiskProcess eq CDD %then %do;
                0 as risk_classification,
                coalesce(cdd.OVERRIDE_SCORE_CLASS_CODE, evt.SCORE_CLASSIFICATION_CODE) as final_disposition_code,
            %end;
            %else %do;
                pty.risk_classification,
                ' ' as final_disposition_code,
            %end;
            pty.politically_exposed_person_ind,
            alt.run_date,
            alt.money_laundering_risk_score,
            pty.employee_ind
      from &segKCSchema..fsk_alert alt
      left join &segKCSchema..fsc_party_dim pty            
      on alt.alerted_entity_number=pty.party_number
         and pty.change_current_ind='Y'
      %if &RiskProcess eq CDD %then %do;
          left join &segKCSchema..fsk_cdd_entity_fact cdd
              on alt.alerted_entity_number=cdd.primary_entity_number
                 and alt.alerted_entity_level_code = cdd.primary_entity_level_code
          left join &segKCSchema..FSK_CDD_SCORING_EVENT evt
              on cdd.LAST_SCORING_EVENT_KEY = evt.CDD_SCORING_EVENT_KEY
      %end;
      where alt.alerted_entity_level_code='PTY'
           and alt.alert_status_code='ACT'
      );

     disconnect from &dbflavor;

 quit;

 %fcf_rcset(&sqlrc);
 %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
     %let _fcfmsg_=ERROR: FCF: Failed to create a temporary table fsk_alert with only PTY alerts;
     %fcf_put_error;
     %goto MACRO_END;
 %end;

 /*------------------------------------------------------------------------------------
  * Find alerted_entity_number and alerted_entity_name for each alert_id in fsk_alert 
  * where alerted_entity_level_code is ASC.  Use the most recent name to prevent 
  * duplicates in the aggregation
  *----------------------------------------------------------------------------------*/
 proc sql;

    connect to &dbflavor as &dbflavor (authdomain="&segkcAuthDomain." &segKCDBConnOpts);

    create table fsk_alert_asc as
    select *
    from connection to &dbflavor

    (select alt.alert_id,
            alt.alerted_entity_level_code,
            alt.alerted_entity_key, 
            alt.alerted_entity_number, 
            assoc.associate_last_name as alerted_entity_name,
            0 as risk_classification,  
            '' as final_disposition_code, 
            '' as politically_exposed_person_ind,
            alt.run_date,
            alt.money_laundering_risk_score,
            alt.employee_ind     /* This is set to Y for associates in fcf_get_entity_info */
     from &segKCSchema..fsk_alert alt
     left join &segKCSchema..fsc_associate_dim assoc
     on alt.alerted_entity_number=assoc.associate_number
        and assoc.change_current_ind='Y'
     where alt.alerted_entity_level_code = 'ASC'
        and alt.alert_status_code='ACT'
     );

    disconnect from &dbflavor;

 quit;
 
 %fcf_rcset(&sqlrc);
 %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
     %let _fcfmsg_=ERROR: FCF: Failed to create a temporary table fsk_alert_asc with only ASC;
     %fcf_put_error;
     %goto MACRO_END;
 %end;
  
 /*------------------------------------------
  * Append asc alerts to PTY alerts  
  *----------------------------------------*/
 proc append base=fsk_alert data=fsk_alert_asc force;  
 run;
%put sqlrc= &syserr;
%put &trans_rc;
 %fcf_rcset(&syserr);
 %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
     %let _fcfmsg_=ERROR: FCF: Failed to append ASC to PTY alerts;
     %fcf_put_error;
     %goto MACRO_END;
 %end;

 /*------------------------------------------------------------------------------------
  * Find alerted_entity_number and alerted_entity_name for each alert_id in fsk_alert 
  * where alerted_entity_level_code is BNK.  Use the most recent name to prevent
  * duplicates in the aggregation
  *----------------------------------------------------------------------------------*/
 proc sql;

    connect to &dbflavor as &dbflavor (authdomain="&segkcAuthDomain." &segKCDBConnOpts);

    create table fsk_alert_bnk as
    select *
    from connection to &dbflavor

    (select alt.alert_id,
            alt.alerted_entity_level_code,
            alt.alerted_entity_key, 
            alt.alerted_entity_number, 
            bnk.bank_name as alerted_entity_name,
            0 as risk_classification,  
            '' as final_disposition_code, 
            '' as politically_exposed_person_ind,
            alt.run_date,
            alt.money_laundering_risk_score,
            alt.employee_ind
     from &segKCSchema..fsk_alert alt
      left join &segKCSchema..fsc_bank_dim bnk
          on alt.alerted_entity_number=bnk.bank_number
             and bnk.change_current_ind='Y'
     where alt.alerted_entity_level_code = 'BNK'
        and alt.alert_status_code='ACT'
     );

    disconnect from &dbflavor;

 quit;

 %fcf_rcset(&sqlrc);
 %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
     %let _fcfmsg_=ERROR: FCF: Failed to create a temporary table fsk_alert_bnk with only BNK alerts;
     %fcf_put_error;
     %goto MACRO_END;
 %end;
  
 /*------------------------------------------
  * Append bnk alerts to PTY and ASC alerts  
  *----------------------------------------*/
 proc append base=fsk_alert data=fsk_alert_bnk force;  
 run;

 %fcf_rcset(&syserr);
 %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
     %let _fcfmsg_=ERROR: FCF: Failed to append BNK to PTY and ASC alerts;
     %fcf_put_error;
     %goto MACRO_END;
 %end;

 /*------------------------------------------------------------------------------------
  * Find alerted_entity_number and alerted_entity_name for each alert_id in fsk_alert 
  * where alerted_entity_level_code is EXT.  Note the external_party_dim does not
  * support type 2 changes so we can't find the most recent name so using the max
  * function to select a common name for all entries with the same name to prevent
  * duplicates in the aggregation
  *----------------------------------------------------------------------------------*/
 proc sql;

    connect to &dbflavor as &dbflavor (authdomain="&segkcAuthDomain." &segKCDBConnOpts);

    create table fsk_alert_ext as
    select *
    from connection to &dbflavor

    (select alert_id,
            alerted_entity_level_code,
            alerted_entity_key, 
            alerted_entity_number, 
            alerted_entity_name,
            0 as risk_classification,  
            '' as final_disposition_code, 
            '' as politically_exposed_person_ind,
            run_date,
            money_laundering_risk_score,
            employee_ind
     from &segKCSchema..fsk_alert alt
     where alt.alerted_entity_level_code = 'EXT'
        and alt.alert_status_code='ACT'
	 order by alerted_entity_number, run_date
     );

    disconnect from &dbflavor;

 quit;

 %fcf_rcset(&sqlrc);
 %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
     %let _fcfmsg_=ERROR: FCF: Failed to create a temporary table fsk_alert_ext with only EXT alerts;
     %fcf_put_error;
     %goto MACRO_END;
 %end;

 /* Since EXT data is not type 2, we need to find the name on the latest date in the alert table and
    use that for all entries with that external party number                                         */

  proc sort data=fsk_alert_ext;
    by alerted_entity_number  run_date;
  run;

 /* find the name on the latest run_date */
 data ext_party_names;
    set fsk_alert_ext;
    by alerted_entity_number  run_date;
    if last.alerted_entity_number;
 run;

%put sqlrc= &syserr;
%put &trans_rc;
 %fcf_rcset(&syserr);
 %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
     %let _fcfmsg_=ERROR: FCF: Failed to create names table for external parties;
     %fcf_put_error;
     %goto MACRO_END;
 %end;

  
 proc sql;

 create table fsk_alert_ext_names as
 select alt.alert_id,
        alt.alerted_entity_level_code,
        alt.alerted_entity_key, 
        alt.alerted_entity_number, 
        extn.alerted_entity_name,
        alt.risk_classification,  
        alt.final_disposition_code, 
        alt.politically_exposed_person_ind,
        alt.run_date,
        alt.money_laundering_risk_score,
        alt.employee_ind
     from fsk_alert_ext alt
     left join ext_party_names extn
	 on alt.alerted_entity_number=extn.alerted_entity_number;
 quit;
     
 
 %fcf_rcset(&sqlrc);
 %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
     %let _fcfmsg_=ERROR: FCF: Failed to create a temporary table fsk_alert_ext_names with only EXT alerts;
     %fcf_put_error;
     %goto MACRO_END;
 %end;

 /*--------------------------------------------------
  * Append ext alerts to party, asc, and bank alerts 
  *------------------------------------------------*/
 proc append base=fsk_alert data=fsk_alert_ext_names force;  
 run;

 %fcf_rcset(&syserr);
 %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
     %let _fcfmsg_=ERROR: FCF: Failed to append EXT alerts to PTY, ASC, and BNK alerts;
     %fcf_put_error;
     %goto MACRO_END;
 %end;

 /*-----------------------------------------------------------------
  * Create a summary table of information about each alerted entity.
  * Transaction counts and aggregate amounts are not available 
  * until the entity transaction table is built.  This table will
  * be updated when the information is available.
  *---------------------------------------------------------------*/
 proc sql;
    create table fsk_alerted_entity_temp as 
    select  
       distinct alerted_entity_level_code, 
       max(alerted_entity_key) as alerted_entity_key ,
       alerted_entity_number,
       alerted_entity_name,
       %if &RiskProcess eq CDD %then %do;
          final_disposition_code as risk_score_code length=32,
       %end;
       %else %do;
          put(risk_classification,1.) as risk_score_code length=32,
       %end;
       count(*) as alerts_cnt, 
       0 as transactions_cnt,
       0 as aggregate_amt,
       (datepart(&fcf_start_datetime) - datepart(min(run_date))) as age_oldest_alert, 
       round(mean(money_laundering_risk_score),1) as money_laundering_score,
       employee_ind,
       politically_exposed_person_ind,
       '' as owner_userid length=240,
       &fcf_start_datetime as created_timestamp
    from fsk_alert
    group by alerted_entity_level_code, alerted_entity_number 
    order by alerted_entity_level_code, alerted_entity_number;
  quit;

  %fcf_rcset(&sqlrc);
  %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
     %let _fcfmsg_=ERROR: FCF: Failed to create summary table fsk_alerted_entity_temp;
     %fcf_put_error;
     %goto MACRO_END;
  %end;

  /*------------------------------------------------
  * Merge the saved routing information
  *------------------------------------------------*/
  data fsk_alerted_entity;
  merge fsk_alerted_entity_temp (in=new) current_routing;
  by alerted_entity_level_code alerted_entity_number;
     
  %if &RiskProcess eq CDD %then %do;
     /* Convert CDD Risk Codes L/M/H to 1/2/3 for Entity Triage UI */
     if compbl(upcase(risk_score_code))='L' then risk_score_code='1';
     else if compbl(upcase(risk_score_code))='M' then risk_score_code='2';
     else if compbl(upcase(risk_score_code))='H' then risk_score_code='3';
     else risk_score_code='0';
  %end;

  if new;

  run;

  %fcf_rcset(&syserr);
  %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
     %let _fcfmsg_=ERROR: FCF: Failed to merge routing information into summary table;
     %fcf_put_error;
     %goto MACRO_END;
  %end;

 /*----------------------------------------------------------------------------
  * If Auto Routing preference set, then Route Entities using Round Robin Method
  *----------------------------------------------------------------------------*/
  %fcf_get_property_value(property=fcf.svr.aml.autoRouteAlertedEntities,
                          output_property_value=AUTO_ROUTE_ENTITY,
                          output_status=property_status); 
  %if &property_status ne 0 %then %do;
      %let _fcf_msg_=WARNING:  The autoRouteAlertedEntities property is not set.  No Auto Routing will be done;
      %let trans_rc=4;
      %fcf_put_error;
  %end;
  %else %if &AUTO_ROUTE_ENTITY ne 0 %then %do;
     %fcf_get_property_value(property=fcf.svr.aml.RouteAlertedEntRole,
                             output_property_value=ROUTE_ENTITY_TO_ROLE,
                             output_status=property_status); 
     %if &property_status ne 0 %then %do;
         %let _fcf_msg_=WARNING:  The RouteAlertedEntRole is not set.  No Auto Routing will be done;
         %let trans_rc=4;
         %fcf_put_error;
     %end; 
     %else %do;
        %fcf_route_entity(inLib=work, 
                          inDS=fsk_alerted_entity, 
                          bank_users=&bankUsersGroup, 
                          role_name=&route_entity_to_role, 
                          column_name=owner_userid,
                          sasdataset=Y
                          ); 

        %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
           %goto MACRO_END;
        %end;
     %end;
  %end;
  %fcf_section_footer(Section=AGP_ROUTE_ENTITY);

 /*------------------------------------------------
  * Replace the existing fsk_alerted_entity table
  *------------------------------------------------*/
 %if %sysfunc(exist(fsk_alerted_entity)) %then %do;
    proc append base=&db_kc_libname..fsk_alerted_entity data=fsk_alerted_entity force nowarn;
    run;  

    %fcf_rcset(&syserr);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: Failed to loading fsk_alerted_entity table;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    %fcf_get_observation_count(table=&db_kc_libname..fsk_alerted_entity);
    %if &fcf_obs_count eq -1 %then %do;
          %let _fcfmsg_=NOTE: FCF: seg_kc.fsk_alerted_entity not exist.;
          %fcf_put_error;
          %put &_fcfmsg_;
          %goto MACRO_END;
    %end;
    %else %if &fcf_obs_count eq 0 %then %do;
          %let _fcfmsg_=NOTE: FCF: No records created for seg_kc.fsk_alerted_entity table.  Skipping execution.;
          %fcf_put_error;
          %goto MACRO_END;
    %end;

    /* Clean-up: Delete the fsk_alerted_entity_temp dataset */
    %put NOTE: FCF: Deleting the fsk_alerted_entity_temp dataset;
    proc datasets library=work nolist;
        delete fsk_alert;
        delete fsk_alert_bnk;
        delete fsk_alert_asc;
        delete fsk_alert_ext;
        delete ext_party_names;
        delete fsk_alert_ext_names;
        delete fsk_alerted_entity_temp;
        delete fsk_alerted_entity;
        delete current_routing;
    quit;
 %end;

 %MACRO_END:

    %if &trans_rc eq 3 or &trans_rc ge 5 %then
       %fcf_execute_macro(fcf_error);
    %else %do;
       %fcf_execute_macro(fcf_success);
    %end;

%mend fcf_ent_load_alerted_entities;
