/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    fcf_fundstracker_create_data.sas
| PRODUCT: SAS Anti-Money Laundering
| SYSTEM:  Windows, Unix
| DESC:    Creates transfer activity data for FundsTracker
|          and stores it in FSK_INT_XFER.
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2016 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro fcf_fundstracker_create_data;

	/*------------
	* Set Module
	*------------*/
	%local _fcfmodule_;
	%let _fcfmodule_=FCF_FUNDSTRACKER_CREATE_DATA;

	/*------------------------------------------
	* Set error code to 0 and message to blank
	*------------------------------------------*/
	%let trans_rc = 0;
	%let _fcfmsg_=;

	/*------------------------------------
	* Get the number of days to look back
	*-------------------------------------*/
	%fcf_get_property_value(property=ft.lookbackDays,
	                  output_property_value=ft_lookback_days,
	                  output_status=ft_lookback_days_rc
	                  );

	%if &ft_lookback_days_rc ne 0 %then %do;
		%let _fcfmsg_=ERROR: FT: Unable to retrieve value of property ft.lookbackDays from FSK_PREFERENCE;
		%let trans_rc = 999999;
		%fcf_put_error;
		%goto MACRO_END;
	%end;

	/*----------------------------------------------
	* Get the minum currency amound for transaction
	*-----------------------------------------------*/
	%fcf_get_property_value(property=ft.minAmount,
	                  output_property_value=ft_min_currency_amt,
	                  output_status=ft_min_currency_amt_rc
	                  );

	%if &ft_min_currency_amt_rc ne 0 %then %do;
		%let _fcfmsg_=ERROR: FT: Unable to retrieve value of property ft.minAmount from FSK_PREFERENCE;
		%let trans_rc = 999999;
		%fcf_put_error;
		%goto MACRO_END;
	%end;

	/*----------------------------------------
	* Get the list of internal transfer types
	*-----------------------------------------*/
	%fcf_get_property_value(property=ft.primaryMedium.internal,
	                  output_property_value=ft_primary_medium_internal,
	                  output_status=ft_primary_medium_internal_rc
	                  );

	%if &ft_primary_medium_internal_rc ne 0 %then %do;
		%let _fcfmsg_=ERROR: FT: Unable to retrieve value of property ft.primaryMedium.internal from FSK_PREFERENCE;
		%let trans_rc = 999999;
		%fcf_put_error;
		%goto MACRO_END;
	%end;


	/*----------------------------------------
	* Get the list of external transfer types
	*-----------------------------------------*/
	%fcf_get_property_value(property=ft.primaryMedium.external,
	                  output_property_value=ft_primary_medium_external,
	                  output_status=ft_primary_medium_external_rc
	                  );

	%if &ft_primary_medium_external_rc ne 0 %then %do;
		%let _fcfmsg_=ERROR: FT: Unable to retrieve value of property ft.primaryMedium.external from FSK_PREFERENCE;
		%let trans_rc = 999999;
		%fcf_put_error;
		%goto MACRO_END;
	%end;

    proc sql;
        %fcf_begin_pass_through(dbflavor=&dbflavor, authenticationDomain=&segkcauthdomain, connOpts=&segKCDBConnOpts);
        execute(DELETE FROM &segKCSchema..FSK_INT_XFER) by &dbflavor;
        %fcf_end_pass_through(dbflavor=&dbflavor, commit=yes);
	quit;

	%fcf_rcset(&sqlrc);
	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FT: Unable to delete contents of FSK_INT_XFER;
		%fcf_put_error;
		%goto MACRO_END;
	%end;

	/*------------------------------------------------------------
	* Calculate lookback date based on runasofdate and preference
	*-------------------------------------------------------------*/

    %if not(%symexist(runasofdate)) %then %do;
        %fcf_get_runasofdate    
    %end;

	data _null_;
		x = input(put(&runasofdate,8.),yymmdd8.) - &ft_lookback_days;
		call symput('lookback',kcompress(put(x,yymmdd10.),'-'));
	run;

	%fcf_rcset(&sysrc);
	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FT: Unable to calculate lookback date;
		%fcf_put_error;
		%goto MACRO_END;
	%end;

	proc sql noprint;
		create table mst_prep.FSK_INT_XFER_U as

		select
			a.account_number as remitter_account_number
			LABEL= 'REMITTER_ACCOUNT_NUMBER',
			ap.party_name as remitter_name,
			ap.party_key as remitter_party_key,
			b.account_number as beneficiary_account_number
			LABEL= 'BENEFICIARY_ACCOUNT_NUMBER',
			bp.party_name as beneficiary_name,
			bp.party_key as beneficiary_party_key,
			c.date_key,
			c.currency_amount,
			'U' as edge_type_code
		from
			&seg_kc_libname..fsc_account_dim a,
			&seg_kc_libname..fsc_account_dim b,
			(
				select
					*
				from
					&seg_kc_libname..fsc_cash_flow_fact
				where
					date_key >= &lookback and
					currency_amount >= &ft_min_currency_amt
			) as c,
			&seg_kc_libname..fsc_transaction_type_dim d,
			&seg_kc_libname..fsc_party_account_bridge apab,
			&seg_kc_libname..fsc_party_account_bridge bpab,
			&seg_kc_libname..fsc_party_dim ap,
			&seg_kc_libname..fsc_party_dim bp
		where
			c.related_ind          = 'N'                     		and
			a.account_key          = c.account_key           		and
			b.account_key          = c.secondary_account_key 		and
			c.transaction_type_key = d.transaction_type_key  		and
			d.primary_medium_desc in (&ft_primary_medium_internal) 	and
			d.transaction_cdi_code = 'D'                     		and
			apab.account_key       = a.account_key           		and
			apab.role_key          = 1                       		and
			apab.change_current_ind = 'Y'                    		and
			ap.party_key           = apab.party_key          		and
			bpab.account_key       = b.account_key           		and
			bpab.role_key          = 1                       		and
			bpab.change_current_ind = 'Y'                    		and
			bp.party_key           = bpab.party_key;
	quit;

	%fcf_rcset(&sqlrc);
	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FT: Unable to create FSK_INT_XFER_U;
		%fcf_put_error;
		%goto MACRO_END;
	%end;

    proc append base = &seg_kc_libname..FSK_INT_XFER
        data = mst_prep.FSK_INT_XFER_U force nowarn;
    run;

	%fcf_rcset(&sysrc);
	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FT: Unable load FSK_INT_XFER_U into FSK_INT_XFER;
		%fcf_put_error;
		%goto MACRO_END;
	%end;

	proc sql noprint;

		create table mst_prep.FSK_INT_XFER_R as

		select
			a.account_number as remitter_account_number,
			ap.party_name as remitter_name,
			ap.party_key as remitter_party_key,
			b.account_number as beneficiary_account_number,
			bp.party_name as beneficiary_name,
			bp.party_key as beneficiary_party_key,
			c.date_key,
			c.currency_amount,
			'R' as edge_type_code
		from
			&seg_kc_libname..fsc_account_dim a,
			&seg_kc_libname..fsc_account_dim b,
			(
				select
					*
				from
					&seg_kc_libname..fsc_cash_flow_fact
				where
					date_key >= &lookback and
					currency_amount >= &ft_min_currency_amt
			) as c,
			&seg_kc_libname..fsc_transaction_type_dim d,
			&seg_kc_libname..fsc_party_account_bridge apab,
			&seg_kc_libname..fsc_party_account_bridge bpab,
			&seg_kc_libname..fsc_party_dim ap,
			&seg_kc_libname..fsc_party_dim bp
		where
			c.related_ind          = 'Y'                     		and
			a.account_key          = c.account_key           		and
			b.account_key          = c.secondary_account_key 		and
			c.transaction_type_key = d.transaction_type_key  		and
			d.primary_medium_desc in (&ft_primary_medium_internal) 	and
			d.transaction_cdi_code = 'D'                     		and
			apab.account_key       = a.account_key           		and
			apab.role_key          = 1                       		and
			apab.change_current_ind = 'Y'                    		and
			ap.party_key           = apab.party_key          		and
			bpab.account_key       = b.account_key           		and
			apab.role_key          = 1                       		and
			bpab.change_current_ind = 'Y'                    		and
			bp.party_key           = bpab.party_key;
	quit;

	%fcf_rcset(&sqlrc);
	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FT: Unable to create FSK_INT_XFER_R;
		%fcf_put_error;
		%goto MACRO_END;
	%end;

	proc append base = &seg_kc_libname..FSK_INT_XFER
        data = mst_prep.FSK_INT_XFER_R force nowarn;
    run;

	%fcf_rcset(&sysrc);
	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FT: Unable load FSK_INT_XFER_R into FSK_INT_XFER;
		%fcf_put_error;
		%goto MACRO_END;
	%end;

	proc sql noprint;

		create table mst_prep.FSK_INT_XFER_O as

		select
			a.account_number as remitter_account_number,
			a.account_name as remitter_name,
			-1 as remitter_party_key,
			f.bank_number || ';' ||e.account_number as beneficiary_account_number,
			e.full_name as beneficiary_name,
			-1 as beneficiary_party_key,
			c.date_key,
			c.currency_amount,
			'O' as edge_type_code
		from
			&seg_kc_libname..fsc_account_dim a,
			(
				select
					*
				from
					&seg_kc_libname..fsc_cash_flow_fact
				where
					date_key >= &lookback and
					currency_amount >= &ft_min_currency_amt
			) as c,
			&seg_kc_libname..fsc_transaction_type_dim d,
			&seg_kc_libname..fsc_ext_party_account_dim e,
			&seg_kc_libname..fsc_bank_dim f
		where
			a.account_key = c.account_key                          and
			c.beneficiary_ext_party_key = e.ext_party_account_key  and
			c.transaction_type_key = d.transaction_type_key        and
			d.primary_medium_desc in (&ft_primary_medium_external) and
			d.transaction_cdi_code = 'D'                           and
			e.bank_key = f.bank_key;
	quit;

	%fcf_rcset(&sqlrc);
	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FT: Unable to create FSK_INT_XFER_O;
		%fcf_put_error;
		%goto MACRO_END;
	%end;

	proc append base = &seg_kc_libname..FSK_INT_XFER
		data = mst_prep.FSK_INT_XFER_O force nowarn;
    run;

	%fcf_rcset(&sysrc);
	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FT: Unable load FSK_INT_XFER_O into FSK_INT_XFER;
		%fcf_put_error;
		%goto MACRO_END;
	%end;

	proc sql noprint;

		create table mst_prep.FSK_INT_XFER_I as

		select
			f.bank_number || ';' ||e.account_number as remitter_account_number,
			e.full_name as remitter_name,
			-1 as remitter_party_key,
			a.account_number as beneficiary_account_number,
			a.account_name as beneficiary_name,
			-1 as beneficiary_party_key,
			c.date_key,
			c.currency_amount,
			'I' as edge_type_code
		from
			&seg_kc_libname..fsc_account_dim a,
			(
				select
					*
				from
					&seg_kc_libname..fsc_cash_flow_fact
				where
					date_key >= &lookback and
					currency_amount >= &ft_min_currency_amt
			) as c,
			&seg_kc_libname..fsc_transaction_type_dim d,
			&seg_kc_libname..fsc_ext_party_account_dim e,
			&seg_kc_libname..fsc_bank_dim f
		where
			a.account_key = c.account_key and
			c.remitter_ext_party_key = e.ext_party_account_key and
			c.transaction_type_key = d.transaction_type_key        and
			d.primary_medium_desc in (&ft_primary_medium_external) and
			d.transaction_cdi_code = 'C'                           and
			e.bank_key = f.bank_key;

	quit;

	%fcf_rcset(&sqlrc);
	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FT: Unable to create FSK_INT_XFER_I;
		%fcf_put_error;
		%goto MACRO_END;
	%end;

	proc append base = &seg_kc_libname..FSK_INT_XFER
		data = mst_prep.FSK_INT_XFER_I force nowarn;
    run;

	%fcf_rcset(&sysrc);
	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FT: Unable load FSK_INT_XFER_I into FSK_INT_XFER;
		%fcf_put_error;
		%goto MACRO_END;
	%end;

	%fcf_get_observation_count(table=&seg_kc_libname..FSK_INT_XFER);

	%MACRO_END:

	%if &trans_rc eq 3 or &trans_rc ge 5 %then
		%fcf_execute_macro(fcf_error);
	%else %do;
		%fcf_execute_macro(post&_fcfmodule_);
		%fcf_execute_macro(fcf_success);
	%end;

%mend;
